import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
/* eslint-disable react/display-name */
/* eslint-disable @typescript-eslint/no-non-null-assertion */
/* eslint-disable react-hooks/exhaustive-deps */
import 'prismjs/themes/prism.css';
import './index.css';
import globalUI from '@sinoform/global-ui';
import useEventCallback from '@sinoui/core/utils/useEventCallback';
import closet from 'dom-helpers/closest';
import contains from 'dom-helpers/contains';
import React, { useCallback, useEffect, useImperativeHandle, useMemo, useRef, useState, } from 'react';
import { Provider, useDispatch, useSelector, useStore } from 'react-redux';
import { Point, Transforms } from 'slate';
import styled from 'styled-components';
import { Toolbar } from '../components/slate-editor';
import { DEFAULT_MENU_HEIGHT, DEFAULT_MENU_WIDTH } from '../constants';
import { ActionType, buildStore, checkSubFieldInRow } from '../reducer';
import calcRange from '../reducer/helper/calcCellRange';
import checkCellMergedInRow from '../reducer/helper/checkCellMergedInRow';
import getTableWidth from '../utils/getTableWidth';
import isInCode from '../utils/isInCode';
import validatesubForm from '../utils/validateSubForm';
import Colgroup from './ColGroup';
import ContextMenu from './ContextMenu';
import drawRange from './drawRange';
import TBody from './TBody';
import THead from './THead';
import usePrintLayout from './usePrintLayout';
const Layout = styled.div `
  position: relative;
  width: 100%;
  height: 100%;
  overflow: hidden;
  font-size: 16px;
`;
const TableContainer = styled.div `
  height: calc(100% - 40px);
  overflow: scroll;
  position: relative;
`;
const Line = styled.div `
  position: absolute;
  top: 0;
  bottom: 0;
  height: ${({ isHorizontal }) => (isHorizontal ? '2px' : '100%')};
  display: none;
  width: ${({ isHorizontal }) => (isHorizontal ? '100%' : '2px')};
  z-index: 9999;
  border-left: ${({ isHorizontal }) => isHorizontal ? 'none' : '1px dotted #000'};
  border-top: ${({ isHorizontal }) => isHorizontal ? '1px dotted #000' : 'none'};
  cursor: ${({ isHorizontal }) => (isHorizontal ? 'row-resize' : 'col-resize')};
`;
function InnerTable({ fields }) {
    const tableRef = useRef(null);
    const { calcCellRange, selectedRange, setSelectedRange, setContextMenuShow, contextMenuShow, copyOrCut, setCopyOrCut, editor, cellRangeForCopyOrCut, setCellRangeForCopyOrCut, setEditor, setEditingCell, } = usePrintLayout();
    const editingCell = useSelector((state) => state.present.editingCell);
    const merges = useSelector((state) => state.present.merges);
    const rows = useSelector((state) => state.present.rows);
    const cols = useSelector((state) => state.present.cols);
    const state = useStore().getState().present;
    const dispatch = useDispatch();
    const [pos, setPos] = useState();
    const windowRef = useRef();
    useEffect(() => {
        windowRef.current = document.querySelector('.sinoui-dialog');
    }, []);
    const handleContextMenu = useCallback((event) => {
        event.preventDefault();
        if (editingCell) {
            return;
        }
        const clickX = event.clientX;
        const clickY = event.clientY;
        const screenW = window.innerWidth;
        const screenH = window.innerHeight;
        let left = clickX;
        let top = clickY;
        if (clickX + DEFAULT_MENU_WIDTH >= screenW) {
            left = clickX - DEFAULT_MENU_WIDTH;
        }
        if (clickY + DEFAULT_MENU_HEIGHT >= screenH) {
            top = clickY - DEFAULT_MENU_HEIGHT;
        }
        setContextMenuShow(true);
        setPos([left, top]);
    }, [setContextMenuShow, editingCell]);
    const changeCellValue = useEventCallback(() => {
        const [rowIdx, colIdx] = editingCell ?? [-1, -1];
        const isValid = validatesubForm(state, rowIdx, colIdx, editor?.children, editor, fields);
        if (isValid) {
            dispatch({
                type: ActionType.EDIT_CELL_VALUE,
                payload: {
                    cellPos: [rowIdx, colIdx],
                    value: editor?.children,
                },
            });
        }
        dispatch({
            type: ActionType.SET_EDITING_CELL,
            payload: undefined,
        });
        if (editor)
            Transforms.setSelection(editor, {
                anchor: { path: [0, 0], offset: 0 },
                focus: { path: [0, 0], offset: 0 },
            });
        setEditor(undefined);
    });
    /**
     * 全局点击事件的处理
     *
     * @param event
     */
    const handleClick = useEventCallback((event) => {
        const enableElement = document.querySelector('.sino-custom-print-layout');
        const targetElement = event.target;
        if (enableElement && !contains(enableElement, targetElement)) {
            changeCellValue();
            windowRef.current?.removeEventListener('click', handleClick, false);
        }
    });
    useEffect(() => {
        tableRef.current?.addEventListener('contextmenu', handleContextMenu);
        return () => {
            tableRef.current?.removeEventListener('contextmenu', handleContextMenu);
        };
    }, [handleContextMenu]);
    const mouseMoveFnRef = useRef(null);
    const mouseUpFnRef = useRef(null);
    const hoverRowIdx = useRef();
    const hoverColIdx = useRef();
    /**
     * 鼠标滑动事件
     *
     * @param startPos 单元格位置
     */
    const handleMouseMove = (startPos) => (event) => {
        // 判断是否是页面的标识线，如果是标识线跳过事件处理方法
        if (event.target?.dataset?.markline) {
            return;
        }
        const target = closet(event.target, 'td');
        if (!target || !target.dataset?.rowid || !target.dataset?.colid) {
            const range = calcRange(startPos, [hoverRowIdx.current, hoverColIdx.current], merges);
            setSelectedRange(range);
            document.removeEventListener('mousemove', mouseMoveFnRef.current);
            document.removeEventListener('mouseup', mouseUpFnRef.current);
            hoverRowIdx.current = undefined;
            hoverColIdx.current = undefined;
            mouseMoveFnRef.current = null;
            mouseUpFnRef.current = null;
            return;
        }
        const rowIdx = target.dataset.rowid
            ? parseInt(target.dataset.rowid, 10)
            : hoverRowIdx.current;
        const colIdx = target.dataset.colid
            ? parseInt(target.dataset.colid, 10)
            : hoverColIdx.current;
        const range = calcRange(startPos, [rowIdx, colIdx], merges);
        hoverRowIdx.current = rowIdx;
        hoverColIdx.current = colIdx;
        drawRange(range[0], range[1]);
    };
    /**
     * 鼠标按起事件
     *
     * @param startPos  单元格位置
     */
    const handleMouseUp = (startPos) => (event) => {
        const target = closet(event.target, 'td');
        if (!target || !target.dataset?.rowid || !target.dataset?.colid) {
            return;
        }
        const rowIdx = parseInt(target.dataset.rowid, 10);
        const colIdx = parseInt(target.dataset.colid, 10);
        calcCellRange(startPos, [rowIdx, colIdx], merges);
        document.removeEventListener('mousemove', mouseMoveFnRef.current);
        document.removeEventListener('mouseup', mouseUpFnRef.current);
        mouseMoveFnRef.current = null;
        mouseUpFnRef.current = null;
    };
    /**
     * 鼠标按下事件
     *
     * @param event  event
     */
    const handleMouseDown = (event) => {
        const target = closet(event.target, 'td');
        if (!target || !target.dataset?.rowid || !target.dataset?.colid) {
            return;
        }
        const rowIdx = parseInt(target.dataset.rowid, 10);
        const colIdx = parseInt(target.dataset.colid, 10);
        const merge = JSON.parse(target.dataset.merge ?? '');
        hoverRowIdx.current = rowIdx;
        hoverColIdx.current = colIdx;
        if (editingCell && editingCell[0] === rowIdx && editingCell[1] === colIdx) {
            return;
        }
        changeCellValue();
        if (event.buttons !== 2) {
            setContextMenuShow(false);
        }
        else if (selectedRange &&
            rowIdx >= selectedRange[0][0] &&
            rowIdx <= selectedRange[1][0] &&
            colIdx >= selectedRange[0][1] &&
            colIdx <= selectedRange[1][1]) {
            // 右键
            // 判断是否在当前选中区域内，如果在当前区域内不做操作，显示右键菜单
            return;
        }
        const endPos = [rowIdx + merge[0], colIdx + merge[1]];
        setSelectedRange([[rowIdx, colIdx], endPos]);
        setEditingCell(undefined);
        drawRange([rowIdx, colIdx], [rowIdx, colIdx]);
        mouseMoveFnRef.current = handleMouseMove([rowIdx, colIdx]);
        mouseUpFnRef.current = handleMouseUp([rowIdx, colIdx]);
        document.addEventListener('mousemove', mouseMoveFnRef.current);
        document.addEventListener('mouseup', mouseUpFnRef.current);
    };
    const handleTdDrop = (event) => {
        const target = closet(event.target, 'td');
        let field = event.dataTransfer.getData('field');
        if (!target || !target.dataset?.rowid || !target.dataset?.colid || !field) {
            return;
        }
        field = JSON.parse(field);
        const rowIdx = parseInt(target.dataset.rowid, 10);
        const colIdx = parseInt(target.dataset.colid, 10);
        if (field.parent && checkCellMergedInRow(rowIdx, merges)) {
            globalUI.message.error('子表单项所在行存在合并单元格的合并行超出当前行');
            return;
        }
        if (editingCell && editingCell[0] === rowIdx && editingCell[1] === colIdx) {
            // 当前单元格处于编辑状态
            const { selection: { anchor, focus }, insertText, } = editor;
            if (Point.equals(anchor, focus)) {
                // 同行不能出现不同子表单的表单项
                if (checkSubFieldInRow(state, [rowIdx, colIdx], [
                    ...(editor?.children ?? []),
                    {
                        type: 'paragraph',
                        children: [{ text: `\${${field.valueExp}}` }],
                    },
                ], fields)) {
                    globalUI.message.error('同一行单元格的表达式中不能出现不同子表单的表单项');
                    return;
                }
                // 判断当前光标是否在表达式内部，如果在表达式内直接插入表单项id，否则需要将表单项id包装成表达式的格式
                const checkIsInCode = isInCode(editor, focus);
                const insertString = checkIsInCode
                    ? `${field.valueExp}`
                    : `\${${field.valueExp}}`;
                insertText(insertString);
                const selectRange = {
                    anchor: {
                        ...anchor,
                        offset: anchor.offset + insertString.length,
                    },
                    focus: { ...focus, offset: focus.offset + insertString.length },
                };
                Transforms.setSelection(editor, selectRange);
            }
            dispatch({
                type: ActionType.EDIT_CELL_VALUE,
                payload: {
                    cellPos: [rowIdx, colIdx],
                    value: editor?.children,
                    fields,
                },
            });
        }
        else {
            // 在当前单元格的最后追加表单项字符串
            const cellValue = rows?.[rowIdx]?.cells?.[colIdx]?.value ?? [];
            const newValue = [
                ...cellValue,
                { type: 'paragraph', children: [{ text: `\${${field.valueExp}}` }] },
            ];
            if (checkSubFieldInRow(state, [rowIdx, colIdx], newValue, fields)) {
                globalUI.message.error('同一行单元格的表达式中不能出现不同子表单的表单项');
                return;
            }
            dispatch({
                type: ActionType.EDIT_CELL_VALUE,
                payload: {
                    cellPos: [rowIdx, colIdx],
                    value: newValue,
                    fields,
                },
            });
        }
    };
    /**
     * 鼠标拖拽悬浮事件
     *
     * @param event  event
     */
    const handleTdDragOver = (event) => {
        const target = closet(event.target, 'td');
        if (target?.dataset?.rowid && target.dataset?.colid) {
            event.preventDefault();
        }
    };
    const handleKeyDown = useEventCallback((event) => {
        if (event.key === 'Escape') {
            changeCellValue();
            document.removeEventListener('keydown', handleKeyDown, false);
        }
    });
    const handleTdDoubleClick = useEventCallback(() => {
        setCellRangeForCopyOrCut(undefined);
        document.addEventListener('keydown', handleKeyDown, false);
        windowRef.current?.addEventListener('click', handleClick, false);
    });
    const style = { width: getTableWidth(cols) };
    return (_jsxs(_Fragment, { children: [_jsx(Toolbar, { editor: editor, selectedRange: selectedRange, editingCell: editingCell }), _jsxs(TableContainer, { children: [_jsxs("table", { className: "custom-print-table", ref: tableRef, onDrop: handleTdDrop, onDragOver: handleTdDragOver, onMouseDown: handleMouseDown, cellPadding: 0, style: style, children: [_jsx(Colgroup, {}), _jsx(THead, { selectedRange: selectedRange }), _jsx(TBody, { selectedRange: selectedRange, handleTdDoubleClick: handleTdDoubleClick, setEditor: setEditor, fields: fields })] }), _jsx(Line, { className: "sinoui-drag-left-line" }), _jsx(Line, { className: "sinoui-drag-right-line" }), _jsx(Line, { isHorizontal: true, className: "sinoui-drag-top-line" }), _jsx(Line, { isHorizontal: true, className: "sinoui-drag-bottom-line" })] }), _jsx(ContextMenu, { pos: pos, contextMenuShow: contextMenuShow, setContextMenuShow: setContextMenuShow, selectedRange: selectedRange, copyOrCut: copyOrCut, setCopyOrCut: setCopyOrCut, cellRangeForCopyOrCut: cellRangeForCopyOrCut, setCellRangeForCopyOrCut: setCellRangeForCopyOrCut, setSelectedRange: setSelectedRange })] }));
}
const Table = React.forwardRef(({ config, fieldItems }, ref) => {
    const store = useMemo(() => buildStore(config), [config]);
    useImperativeHandle(ref, () => ({
        getCurrentConfig: () => store.getState().present,
        replaceConfig: (value) => store.dispatch({ type: ActionType.REPLACE_CONFIG, payload: value }),
    }));
    return (_jsx(Provider, { store: store, children: _jsx(Layout, { children: _jsx(InnerTable, { fields: fieldItems }) }) }));
});
export default Table;
