import isEqual from 'lodash/isEqual';
import { DEFAULT_CELL_VALUE, PAGE_SIZE_CONFIG, getMaxColLenInPageConfig, mmToPx, } from '@sinoform/print-out-core';
/**
 * 校验是否有单元格超出了设置的页面边距
 * @param customPrintConfig 自定义布局配置信息
 * @returns
 */
export default function checkConfigIsOutRange(customPrintConfig) {
    if (!customPrintConfig) {
        return false;
    }
    const { rows, cols, config = {} } = customPrintConfig;
    const { pageConfig } = config;
    const { pageDir, pageSize, pagePaddingLeft, pagePaddingRight } = pageConfig;
    const paddingLeft = parseFloat(pagePaddingLeft);
    const paddingRight = parseFloat(pagePaddingRight);
    const pageWidth = mmToPx(PAGE_SIZE_CONFIG[pageSize]['v'][pageDir === '纵向' ? 0 : 1] -
        (paddingLeft + paddingRight));
    const maxCol = getMaxColLenInPageConfig(pageWidth, cols);
    for (let i = 0; i < rows.len; i += 1) {
        for (let j = 0; j < cols.len; j += 1) {
            if (rows[i]?.cells?.[j]) {
                const { value, merge = [0, 0] } = rows[i].cells[j];
                // 分两种场景的超出
                // 1. 合并单元格的起始位置未超出，但是结束位置超出
                // 2. 非合并的单元格位置位置超出，且单元格内容不为空
                if ((j < maxCol && j + merge[1] >= maxCol) ||
                    (value && !isEqual(value, DEFAULT_CELL_VALUE) && j >= maxCol)) {
                    return true;
                }
            }
        }
    }
    return false;
}
