/* eslint-disable no-param-reassign */
import { Editor, Transforms, Range, Point } from 'slate';
import getNodeInfoByPoint from './getNodeInfoByPoint';
import isCodeType from './isCodeType';
export default function withShortcuts(editor) {
    const { insertText, deleteBackward, deleteForward } = editor;
    editor.insertText = (text) => {
        const { selection } = editor;
        if (text === '{' && selection && Range.isCollapsed(selection)) {
            if (isCodeType(editor, selection.focus)) {
                insertText(text);
                return;
            }
            const { anchor } = selection;
            const block = Editor.above(editor, {
                match: (n) => Editor.isBlock(editor, n),
            });
            const path = block ? block[1] : [];
            const start = Editor.start(editor, path);
            const range = { anchor, focus: start };
            const beforeText = Editor.string(editor, range);
            if (beforeText.endsWith('$')) {
                insertText('{}');
                const selectRange = {
                    anchor: { ...selection.anchor, offset: selection.anchor.offset + 1 },
                    focus: { ...selection.focus, offset: selection.focus.offset + 1 },
                };
                Transforms.setSelection(editor, selectRange);
                return;
            }
        }
        insertText(text);
    };
    editor.deleteBackward = (...args) => {
        const { selection: { anchor, focus }, } = editor;
        if (Point.equals(anchor, focus) && isCodeType(editor, focus)) {
            const focusCodeBlock = getNodeInfoByPoint(editor, focus);
            if (focus.offset === focusCodeBlock.end) {
                Transforms.select(editor, {
                    anchor: { ...anchor, offset: focusCodeBlock.start },
                    focus: { ...focus, offset: focusCodeBlock.end },
                });
                return;
            }
        }
        deleteBackward(...args);
    };
    editor.deleteForward = (...args) => {
        const { selection: { anchor, focus }, } = editor;
        if (Point.equals(anchor, focus) && isCodeType(editor, focus)) {
            const focusCodeBlock = getNodeInfoByPoint(editor, focus);
            if (focus.offset === focusCodeBlock.start) {
                Transforms.select(editor, {
                    anchor: { ...anchor, offset: focusCodeBlock.start },
                    focus: { ...focus, offset: focusCodeBlock.end },
                });
                return;
            }
        }
        deleteForward(...args);
    };
    return editor;
}
