"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.changeColWidth = changeColWidth;
exports.changeRowHeight = changeRowHeight;
exports.copyCell = copyCell;
exports.cutCell = cutCell;
exports.deleteCol = deleteCol;
exports.deleteRow = deleteRow;
exports.insertCol = insertCol;
exports.insertRow = insertRow;
exports.mergeCells = mergeCells;
exports.setCellsBorderStyle = setCellsBorderStyle;
exports.setCellsStyle = setCellsStyle;
exports.setPrintConfig = setPrintConfig;
var _types = /*#__PURE__*/require("./types");
/**
 * 合并或拆分单元格
 *
 * 如果选中的单元格中含有被合并的单元格，执行拆分动作，拆分选中区域内所有的被合并的单元格，否则执行合并操作
 *
 * 前提：确保选中单元格的边界是完整的单元格，不能出现被合并的单元格一部分再选中区域内，一部分在选中区域外的情况
 *
 * @param fromCell 选中单元格的起始位置
 * @param toCell   选中单元格的结束位置
 * @returns
 */
function mergeCells(fromCell, toCell) {
  // 确保起始和结束节点的位置是从左上到右下的方向
  return {
    type: _types.ActionType.MERGE_CELLS,
    payload: {
      fromCell: [Math.min(fromCell[0], toCell[0]), Math.min(fromCell[1], toCell[1])],
      toCell: [Math.max(fromCell[0], toCell[0]), Math.max(fromCell[1], toCell[1])]
    }
  };
}
/**
 * 拖动改变行高
 * @param rowIdx
 * @param height
 * @returns
 */
function changeRowHeight(rowIdx, height) {
  return {
    type: _types.ActionType.CHANGE_ROW_HEIGHT,
    payload: {
      rowIdx: rowIdx,
      height: height
    }
  };
}
/**
 * 拖动改变列宽
 * @param colIdx
 * @param width
 * @returns
 */
function changeColWidth(colIdx, width) {
  return {
    type: _types.ActionType.CHANGE_COL_WIDTH,
    payload: {
      colIdx: colIdx,
      width: width
    }
  };
}
/**
 * 新增行
 * @param rowIdx
 * @returns
 */
function insertRow(rowIdx) {
  return {
    type: _types.ActionType.INSERT_ROW,
    payload: rowIdx
  };
}
/**
 * 删除行
 * @param from
 * @param len
 * @returns
 */
function deleteRow(from, len) {
  return {
    type: _types.ActionType.DELETE_ROW,
    payload: {
      from: from,
      len: len
    }
  };
}
/**
 * 新增列
 * @param colIdx
 * @returns
 */
function insertCol(colIdx) {
  return {
    type: _types.ActionType.INSERT_COL,
    payload: colIdx
  };
}
/**
 * 删除选中的列
 * @param startCol  选中单元格的起始列
 * @param endCol    选中单元格的结束列
 */
function deleteCol(startCol, endCol) {
  return {
    type: _types.ActionType.DELETE_COL,
    payload: {
      startCol: Math.min(startCol, endCol),
      endCol: Math.max(startCol, endCol)
    }
  };
}
/**
 * 单元格复制
 *
 * @param source 选中的单元格起始和结束位置
 * @param target 目标单元格位置
 * @returns
 */
function copyCell(source, target) {
  return {
    type: _types.ActionType.COPY_CELL,
    payload: {
      source: source,
      target: target
    }
  };
}
function cutCell(source, target) {
  return {
    type: _types.ActionType.CUT_CELL,
    payload: {
      source: source,
      target: target
    }
  };
}
function setPrintConfig(config) {
  return {
    type: _types.ActionType.SET_PRINT_CONFIG,
    payload: config
  };
}
function setCellsBorderStyle(range, type) {
  var style = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : ['thin', '#000'];
  return {
    type: _types.ActionType.SET_CELLS_BORDER_STYLE,
    payload: {
      range: range,
      type: type,
      style: style
    }
  };
}
function setCellsStyle(cells, style) {
  return {
    type: _types.ActionType.SET_CELLS_STYLE,
    payload: {
      cells: cells,
      style: style
    }
  };
}