"use strict";

var _interopRequireDefault = /*#__PURE__*/require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.checkCellRange = exports.canPastCell = void 0;
var _slicedToArray2 = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@babel/runtime/helpers/slicedToArray"));
/**
 * 校验复制的区域粘贴到目标区域之后是否超出了单元格边界
 * @param sourceRange
 * @param targetPos
 * @param draft
 */
var canPastCell = function canPastCell(sourceRange, targetPos, draft) {
  var rows = draft.rows,
    cols = draft.cols;
  var _targetPos = (0, _slicedToArray2.default)(targetPos, 2),
    tri = _targetPos[0],
    tci = _targetPos[1];
  var _sourceRange = (0, _slicedToArray2.default)(sourceRange, 2),
    _sourceRange$ = (0, _slicedToArray2.default)(_sourceRange[0], 2),
    sri = _sourceRange$[0],
    sci = _sourceRange$[1],
    _sourceRange$2 = (0, _slicedToArray2.default)(_sourceRange[1], 2),
    eri = _sourceRange$2[0],
    eci = _sourceRange$2[1];
  return tri + eri - sri < rows.len && tci + eci - sci < cols.len;
};
/**
 * 目标位置对应的区域是否满足源选中区域的单元格粘贴的条件
 *
 * 即对应区域的单元格是否有合并的单元格超出边界的情况
 *
 * @param sourceRange 源选中单元格区域
 * @param targetPos   目标位置
 * @param draft       当前布局状态
 */
exports.canPastCell = canPastCell;
var checkCellRange = function checkCellRange(targetPos, sourceRange) {
  var merges = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  var _sourceRange2 = (0, _slicedToArray2.default)(sourceRange, 2),
    _sourceRange2$ = (0, _slicedToArray2.default)(_sourceRange2[0], 2),
    sri = _sourceRange2$[0],
    sci = _sourceRange2$[1],
    _sourceRange2$2 = (0, _slicedToArray2.default)(_sourceRange2[1], 2),
    eri = _sourceRange2$2[0],
    eci = _sourceRange2$2[1];
  // 计算出对应的目标单元格区域
  var _targetPos2 = (0, _slicedToArray2.default)(targetPos, 2),
    tsri = _targetPos2[0],
    tsci = _targetPos2[1];
  var teri = tsri + eri - sri;
  var teci = tsci + eci - sci;
  return !merges.some(function (_ref) {
    var _ref2 = (0, _slicedToArray2.default)(_ref, 2),
      _ref2$ = (0, _slicedToArray2.default)(_ref2[0], 2),
      msri = _ref2$[0],
      msci = _ref2$[1],
      _ref2$2 = (0, _slicedToArray2.default)(_ref2[1], 2),
      meri = _ref2$2[0],
      meci = _ref2$2[1];
    return (
      // 上方的单元格被合并超出上边界
      msri < tsri && meri >= tsri && msci > tsci && msci <= teci ||
      // 左方的单元格被合并超出左边界
      msci < tsci && meci >= tsci && msri > tsri && msri <= teri ||
      // 下方的单元格被合并并超出下边界
      msri <= teri && meri > teri && msci >= tsci && msci <= teci ||
      // 右方的单元格被合并并超出右边界
      msci <= teci && meci > teci && msri >= tsri && msri <= teri
    );
  });
};
exports.checkCellRange = checkCellRange;