"use strict";

var _interopRequireDefault = /*#__PURE__*/require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = autoSelectCodeBlock;
var _objectSpread2 = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@babel/runtime/helpers/objectSpread2"));
var _slate = /*#__PURE__*/require("slate");
var _getNodeInfoByPoint = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("./getNodeInfoByPoint"));
var _isCodeType = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("./isCodeType"));
/**
 * 自动选中代码块
 *
 * @param editor 编辑器状态
 */
function autoSelectCodeBlock(editor) {
  if (!editor || !editor.selection) {
    return;
  }
  var _editor$selection = editor.selection,
    anchor = _editor$selection.anchor,
    focus = _editor$selection.focus;
  var anchorRow = anchor.path[0];
  var anchorCol = anchor.path[1];
  var focusRow = focus.path[0];
  var focusCol = focus.path[1];
  var anchorInfo = (0, _getNodeInfoByPoint.default)(editor, anchor);
  var focusInfo = (0, _getNodeInfoByPoint.default)(editor, focus);
  if ((0, _isCodeType.default)(editor, anchor)) {
    // 从前往后选择，锚点所在元素必须有选中
    if ((anchorRow < focusRow || anchorCol < focusCol || anchor.offset < focus.offset) && anchor.offset < anchorInfo.end) {
      // 如果锚点不是代码块的起点，焦点是代码块且不是代码块的末尾，则自动选中开始代码块和结束代码块
      if (anchor.offset !== 0 && (0, _isCodeType.default)(editor, focus) && focus.offset !== focusInfo.end && focus.offset > focusInfo.start) {
        var newSelect = {
          anchor: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, anchor), {}, {
            offset: anchorInfo.start
          }),
          focus: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, focus), {}, {
            offset: focusInfo.end
          })
        };
        _slate.Transforms.select(editor, newSelect);
        return;
      }
      // 如果锚点不是代码块起点，焦点为任意元素，则自动选中开始代码块
      if (anchor.offset !== 0) {
        var _newSelect = {
          anchor: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, anchor), {}, {
            offset: anchorInfo.start
          }),
          focus: focus
        };
        _slate.Transforms.select(editor, _newSelect);
        return;
      }
      // 如果焦点所在元素是代码块，且不是代码块的末端，则自动选中结束代码块
      if ((0, _isCodeType.default)(editor, focus) && focus.offset !== focusInfo.end) {
        var _newSelect2 = {
          anchor: anchor,
          focus: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, focus), {}, {
            offset: focusInfo.end
          })
        };
        _slate.Transforms.select(editor, _newSelect2);
      }
    } else if (
    // 从后往前选择,且必须有元素选中
    (anchorRow > focusRow || anchorCol > focusCol || anchor.offset > focus.offset) && focus.offset < anchorInfo.end && anchor.offset > anchorInfo.start) {
      // 如果锚点不是代码块的末尾，焦点所在元素为代码块且不是代码块的开始,则自动选中锚点和焦点所在的代码块
      if (anchor.offset !== anchorInfo.end && (0, _isCodeType.default)(editor, focus) && focus.offset !== focusInfo.start) {
        var _newSelect3 = {
          anchor: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, anchor), {}, {
            offset: anchorInfo.end
          }),
          focus: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, focus), {}, {
            offset: focusInfo.start
          })
        };
        _slate.Transforms.select(editor, _newSelect3);
        return;
      }
      // 如果锚点不是所在代码块的末尾，焦点为普通元素，则自动选中锚点所在代码块
      if (anchor.offset !== anchorInfo.end) {
        var _newSelect4 = {
          anchor: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, anchor), {}, {
            offset: anchorInfo.end
          }),
          focus: focus
        };
        _slate.Transforms.select(editor, _newSelect4);
        return;
      }
      // 如果焦点所在元素为代码块，且焦点所在位置不是代码块的起点，则自动选中焦点所在代码块
      if ((0, _isCodeType.default)(editor, focus) && focus.offset !== focusInfo.start) {
        var _newSelect5 = {
          anchor: anchor,
          focus: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, focus), {}, {
            offset: focusInfo.start
          })
        };
        _slate.Transforms.select(editor, _newSelect5);
      }
    }
  } else if ((0, _isCodeType.default)(editor, focus)) {
    // 从前往后选择，锚点为任意元素，焦点为表达式且未全部选中，则自动选中焦点代码块
    if ((anchorRow < focusRow || anchorCol < focusCol || anchor.offset < focus.offset) && focus.offset > focusInfo.start && focus.offset !== focusInfo.end) {
      var _newSelect6 = {
        anchor: anchor,
        focus: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, focus), {}, {
          offset: focusInfo.end
        })
      };
      _slate.Transforms.select(editor, _newSelect6);
      return;
    }
    // 从后往前选择，焦点所在位置不是代码块的开始，则自动选中焦点所在代码块
    if ((anchorRow > focusRow || anchorCol > focusCol || anchor.offset > focus.offset) && focus.offset !== focusInfo.start && focus.offset < focusInfo.end) {
      var _newSelect7 = {
        anchor: anchor,
        focus: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, focus), {}, {
          offset: focusInfo.start
        })
      };
      _slate.Transforms.select(editor, _newSelect7);
    }
  }
}