"use strict";

var _interopRequireDefault = /*#__PURE__*/require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = setCellTextStyleHelper;
var _defineProperty2 = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@babel/runtime/helpers/defineProperty"));
var _objectSpread3 = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@babel/runtime/helpers/objectSpread2"));
var _slicedToArray2 = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("@babel/runtime/helpers/slicedToArray"));
var _printOutCore = /*#__PURE__*/require("@sinoform/print-out-core");
var _reducer = /*#__PURE__*/require("../reducer");
var _CustomEditor = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("./CustomEditor"));
var _getAllSelectedCells = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("./getAllSelectedCells"));
/**
 * 设置单元格文本样式的辅助函数
 *
 * @param editor 编辑器
 * @param dispatch
 * @param styleType 样式类型
 * @param styleValue 样式的值
 * @param rows 行数据
 * @param rows.len
 * @param merges 合并单元格集合
 * @param selectedRange 选中范围
 * @param editingCell 处于编辑状态的单元格
 * @returns
 */
function setCellTextStyleHelper(editor, dispatch, styleType, styleValue, rows) {
  var merges = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : [];
  var selectedRange = arguments.length > 6 ? arguments[6] : undefined;
  var editingCell = arguments.length > 7 ? arguments[7] : undefined;
  if (!selectedRange) {
    return;
  }
  if (editingCell) {
    switch (styleType) {
      case 'bold':
        _CustomEditor.default.toggleBoldMark(editor);
        break;
      case 'underLine':
        _CustomEditor.default.toggleUnderLineMark(editor);
        break;
      case 'deleteLine':
        _CustomEditor.default.toggleDeleteLineMark(editor);
        break;
      case 'italic':
        _CustomEditor.default.toggleItalicMark(editor);
        break;
      case 'color':
        _CustomEditor.default.changeTextColor(editor, styleValue);
        break;
      case 'fontSize':
        _CustomEditor.default.toggeleFontSize(editor, styleValue);
        break;
      case 'fontFamily':
        _CustomEditor.default.toggeleFontFamily(editor, styleValue);
        break;
      default:
        break;
    }
  } else {
    var selectedCells = (0, _getAllSelectedCells.default)(selectedRange, merges);
    var items = selectedCells.map(function (_ref) {
      var _rows$rowIdx$cells$co, _rows$rowIdx, _rows$rowIdx$cells$co2;
      var _ref2 = (0, _slicedToArray2.default)(_ref, 2),
        rowIdx = _ref2[0],
        colIdx = _ref2[1];
      var cellValue = (_rows$rowIdx$cells$co = (_rows$rowIdx = rows[rowIdx]) === null || _rows$rowIdx === void 0 ? void 0 : (_rows$rowIdx$cells$co2 = _rows$rowIdx.cells[colIdx]) === null || _rows$rowIdx$cells$co2 === void 0 ? void 0 : _rows$rowIdx$cells$co2.value) !== null && _rows$rowIdx$cells$co !== void 0 ? _rows$rowIdx$cells$co : _printOutCore.DEFAULT_CELL_VALUE;
      return cellValue.map(function (item) {
        return (0, _objectSpread3.default)((0, _objectSpread3.default)({}, item), {}, {
          children: item.children.map(function (textItem) {
            return (0, _objectSpread3.default)((0, _objectSpread3.default)({}, textItem), {}, (0, _defineProperty2.default)({}, styleType, styleType === 'color' || styleType === 'fontSize' || styleType === 'fontFamily' ? styleValue : !textItem[styleType]));
          })
        });
      });
    });
    dispatch({
      type: _reducer.ActionType.SET_CELLS_TEXT_STYLE,
      payload: {
        cells: selectedCells,
        items: items
      }
    });
  }
}